﻿
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using Ozeki.MediaGateway;
using Ozeki.MediaGateway.Service;
using Ozeki.VoIP;
using Ozeki.VoIP.SDK;

namespace SIPGateway
{
    /// <summary>
    /// Implements the SIP softphone service.
    /// </summary>
    class SIPGateway : MediaGateway
    {
        Dictionary<IClient, SIPClient> sipClients;
        ISoftPhone softPhone;
        IStreamService streamService;
        SIPAccountPool sipAccountPool;

        /// <summary>
        /// Creates a new instance of SIPGateway.
        /// </summary>
        public SIPGateway()
        {
            Console.WriteLine("SIP Gateway starting...");
        }

        /// <summary>
        /// Indicates when the mediaGateway successfully started and this time It initialize softphone and other things.
        /// </summary>
        public override void OnStart()
        {
            Console.WriteLine("SIP Gateway started");

            sipClients = new Dictionary<IClient, SIPClient>();
            softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP(), 5060, 5800, 5060);

            sipAccountPool = new SIPAccountPool();

            for (int i = 80; i < 99; i++)
            {
                var name = "oz8" + i;
                var sipAccount = new SIPAccount(true, name, name, name, name, "192.168.112.100", 5060);

                sipAccountPool.AddSIPAcount(sipAccount);
            }

            streamService = GetService<IStreamService>();
        }

        /// <summary>
        /// Indicates the client connection and registers it if the chatclient dictionary doesn't contain the client nickname.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="parameters">Opional parameters</param>
        public override void OnClientConnect(IClient client, object[] parameters)
        {
            Console.WriteLine("New client connected. Remote address: {0}", client.RemoteAddress);

            var sipAccount = sipAccountPool.GetSIPAccount();

            if(sipAccount == null)
            {
                client.InvokeMethod("OnRegistrationStateChanged", RegistrationState.ConnectionLimit);
                client.Close();
                return;
            }

            var sipClient = new SIPClient(client, streamService, softPhone, sipAccount);
            sipClients.Add(client, sipClient);

            sipClient.Register();
        }

        /// <summary>
        ///  Indicates client disconnection and remove client's keyvaluepair from chatclient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public override void OnClientDisconnect(IClient client)
        {
            Console.WriteLine("Client disconnected");

            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out sipClient))
                {
                    sipClient.Unregister();
                    sipAccountPool.AddSIPAcount(sipClient.Account);
                    sipClients.Remove(client);
                }
            }

        }

        /// <summary>
        /// Indicates the client starts publish his IMediastream.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="mediaStream">Published IMediaStream</param>
        public override void OnStreamPublishStart(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("Media stream published. Stream name {0}", mediaStream.Name);

            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out sipClient))
                    sipClient.StreamPublished(mediaStream);
            }

        }

        /// <summary>
        /// Indicates the client close his IMediaStream.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="mediaStream">Closed IMediaStream</param>
        public override void OnStreamClose(IClient client, IMediaStream mediaStream)
        {
            Console.WriteLine("Media stream closed.");

            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out sipClient))
                    sipClient.StreamClosed();
            }

        }

        /// <summary>
        /// Forward the call request to the SIPclient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="phoneNumber">dial number</param>
        public void Call(IClient client, string phoneNumber)
        {
            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out sipClient))
                    sipClient.Call(phoneNumber);
            }

        }

        /// <summary>
        /// Forward the hangup request to the SIPclient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void HangUp(IClient client)
        {
            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out sipClient))
                    sipClient.HangUp();
            }

        }

        /// <summary>
        /// Forwards the Accept request to the SIPclient.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void Accept(IClient client)
        {
            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out sipClient))
                    sipClient.Accept();
            }

        }

        /// <summary>
        /// Reject an incoming call request.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        public void Reject(IClient client)
        {
            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out  sipClient))
                    sipClient.Reject();
            }

        }

        /// <summary>
        /// Indicates the client starts to send  a dtmf sign.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="dtmf">The sended Dtmf sign.</param>
        public void StartDtmf(IClient client, DtmfNamedEvents dtmf)
        {
            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out  sipClient))
                    sipClient.StartDtmf(dtmf);
            }

        }
        /// <summary>
        ///  Indicates the client stops to send  the dtmf sign.
        /// </summary>
        /// <param name="client">That client reference who call this method.</param>
        /// <param name="dtmf">The sended Dtmf sign.</param>
        public void StopDtmf(IClient client, DtmfNamedEvents dtmf)
        {
            lock (sipClients)
            {
                SIPClient sipClient;
                if (sipClients.TryGetValue(client, out  sipClient))
                    sipClient.StopDtmf(dtmf);
            }

            
        }


    }
}
